/* tree' utility for printing dir content as a tree.
(c) 2002,2003,2004
Stanislav Ievlev <inger@altlinux.org>
License: GPL
*/

#ifndef TREE_H

#include <dirent.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/param.h>

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <signal.h>

#include <errno.h>
#include <error.h>

#include <getopt.h>
#include <libgen.h>
#include <regex.h>

#include <locale.h>

/*graphics definitions from pstree package*/

/* UTF-8 defines by Johan Myreen, updated by Ben Winslow */
#define UTF_V	"\342\224\202"	/* U+2502, Vertical line drawing char */
#define UTF_VR	"\342\224\234"	/* U+251C, Vertical and right */
#define UTF_H	"\342\224\200"	/* U+2500, Horizontal */
#define UTF_UR	"\342\224\224"	/* U+2514, Up and right */
#define UTF_HD	"\342\224\254"	/* U+252C, Horizontal and down */

#define VT_BEG	"\033(0\017"	/* use graphic chars */
#define VT_END	"\033(B"	/* back to normal char set */
#define VT_V	"x"		/* see UTF definitions above */
#define VT_VR	"t"
#define VT_H	"q"
#define VT_UR	"m"
#define	VT_HD	"w"

typedef struct
{
	const char *empty;	/*   */
	const char *vertical;	/* | */
	const char *horizontal;	/* - */
	const char *branch;	/* |- */
	const char *coner;	/* ` */
}
graphics;

static graphics sym_ascii = { " ", "|", "-", "|", "`" }, sym_vt100 =
{
" ", VT_BEG VT_V VT_END,
		VT_BEG VT_H VT_END, VT_BEG VT_VR VT_END, VT_BEG VT_UR VT_END,}
, sym_utf8 =
{
" ", UTF_V, UTF_H, UTF_VR, UTF_UR,};

/*token names for colorification*/
enum
{
	REGULAR = 0,
	EXECUTE,
	DIRECTORY,
	LINK_OK,
	LINK_BAD,
	BLOCK,
	CHAR,
	LEFTCODE,
	RIGHTCODE,
	NORMAL,
	FIFO,
	SOCKET,
	DUMMY
};

enum
{
	MAX_TOKEN = 15
};

/*default token settings*/
char    tokens[DUMMY + 1][MAX_TOKEN] = {
	{"00"}, /*REGULAR*/ {"01;32"}, /*EXECUTE*/ {"01;34"}, /*DIRECTORY*/ {"01;36"},	/*LINK_OK */
	{"01;05;37;41"},	/*LINK_BAD */
	{"40;33;01"}, /*BLOCK*/
	{"40;33;01"}, /*CHAR*/
	{"\033["}, /*LEFTCODE*/
	{"m"}, /*RIGHTCODE*/
	{"00"}, /*NORMAL*/ {"40;33"}, /*FIFO*/ {"01;35"} /*SOCKET*/
};

/*extern declaration*/
void    colorize (const char *, int);

/*helpers*/
void   *
xmalloc (size_t s)
{
	void   *tmp = malloc (s);

	if (!tmp)
	{
		perror ("malloc");
		exit (0);
	} else
	{
		return tmp;
	}
}

void *
xrealloc (void *ptr, size_t s)
{
	ptr = realloc(ptr, s);

	if (!ptr)
	{
		perror("realloc");
		exit(1);
	}
	return ptr;
}

void
xfree (void *ptr)
{
	if (!ptr)
	{
		error(1, 0, "unexpected NULL pointer");
	}
	free(ptr);
}

#endif
